/**
 * WebSocket工具类封装
 * 提供更简单的API接口，方便业务层使用
 */

import WebSocketClient from './websocket';

// WebSocket实例
let wsInstance = null;

// 事件监听器集合
const eventListeners = {};

/**
 * WebSocket工具类
 */
const WebSocketUtil = {
  /**
   * 初始化WebSocket连接
   * @param {Object} options 配置项
   * @param {String} options.url WebSocket连接地址
   * @param {Number} options.heartbeatInterval 心跳间隔时间(ms)，默认30000ms
   * @param {Number} options.reconnectInterval 重连间隔时间(ms)，默认5000ms
   * @param {Number} options.reconnectMaxTimes 最大重连次数，默认10次
   */
  init(options) {
    if (wsInstance) {
      console.warn('WebSocket已初始化，请勿重复初始化');
      return;
    }
    
    wsInstance = new WebSocketClient({
      ...options,
      onOpen: (event) => {
        // console.log('WebSocket连接成功');
        this.dispatchEvent('open', event);
      },
      onMessage: (data, event) => {
        // console.log('WebSocket收到消息:', data);
        this.dispatchEvent('message', { data, event });
      },
      onError: (event) => {
        // console.error('WebSocket连接错误:', event);
        this.dispatchEvent('error', event);
      },
      onClose: (event) => {
        // console.log('WebSocket连接关闭:', event);
        this.dispatchEvent('close', event);
      }
    });
  },
  
  /**
   * 连接WebSocket
   * @param {String} url 连接地址，可选，如果不传则使用初始化时的url
   */
  connect(url) {
    if (!wsInstance) {
      if (!url) {
        console.log('WebSocket未初始化，请先调用init方法');
        return;
      }
      
      // 如果未初始化，则先初始化
      this.init({ url });
      return;
    }
    
    wsInstance.connect(url);
  },
  
  /**
   * 发送消息
   * @param {Object|String} data 要发送的数据
   * @returns {Boolean} 是否发送成功
   */
  send(data) {
    if (!wsInstance) {
      console.log('WebSocket未初始化，请先调用init方法');
      return false;
    }
    
    return wsInstance.send(data);
  },
  
  /**
   * 关闭连接
   */
  close() {
    if (!wsInstance) {
      console.warn('WebSocket未初始化，无需关闭');
      return;
    }
    
    wsInstance.close();
    wsInstance = null;
  },
  
  /**
   * 获取当前连接状态
   * @returns {Number} 0-未连接，1-连接中，2-已连接
   */
  getStatus() {
    if (!wsInstance) {
      return 0;
    }
    
    return wsInstance.getStatus();
  },
  
  /**
   * 添加事件监听
   * @param {String} event 事件名称，支持：open, message, error, close
   * @param {Function} callback 回调函数
   */
  addEventListener(event, callback) {
    if (!eventListeners[event]) {
      eventListeners[event] = [];
    }
    
    eventListeners[event].push(callback);
  },
  
  /**
   * 移除事件监听
   * @param {String} event 事件名称
   * @param {Function} callback 回调函数，如果不传则移除该事件的所有监听
   */
  removeEventListener(event, callback) {
    if (!eventListeners[event]) {
      return;
    }
    
    if (!callback) {
      // 移除该事件的所有监听
      eventListeners[event] = [];
      return;
    }
    
    // 移除指定回调
    const index = eventListeners[event].indexOf(callback);
    if (index !== -1) {
      eventListeners[event].splice(index, 1);
    }
  },
  
  /**
   * 触发事件
   * @param {String} event 事件名称
   * @param {*} data 事件数据
   */
  dispatchEvent(event, data) {
    if (!eventListeners[event]) {
      return;
    }
    
    eventListeners[event].forEach(callback => {
      try {
        callback(data);
      } catch (error) {
        console.log(`WebSocket事件${event}回调执行错误:`, error);
      }
    });
  }
};

export default WebSocketUtil;