# WebSocket工具类使用说明

## 简介

本工具类提供了WebSocket连接管理、自动重连、心跳检测等功能，可以方便地在Vue项目中使用WebSocket进行实时通信。

## 文件结构

- `websocket.js`: WebSocket核心实现类，提供WebSocket连接管理、自动重连、心跳检测等功能
- `WebSocketUtil.js`: WebSocket工具类封装，提供更简单的API接口，方便业务层使用
- `WebSocketDemo.vue`: WebSocket使用示例组件

## 功能特性

1. **连接管理**：自动建立连接、关闭连接
2. **自动重连**：连接断开后自动重连，可配置重连间隔和最大重连次数
3. **心跳检测**：定时发送心跳消息，保持连接活跃
4. **事件监听**：支持监听连接打开、消息接收、错误、关闭等事件
5. **消息发送**：支持发送文本消息和JSON对象

## 使用方法

### 1. 初始化WebSocket

```javascript
import WebSocketUtil from '@/utils/WebSocketUtil';

// 初始化WebSocket
WebSocketUtil.init({
  url: 'ws://your-websocket-server.com',  // WebSocket服务地址
  heartbeatInterval: 30000,               // 心跳间隔，默认30秒
  reconnectInterval: 5000,                // 重连间隔，默认5秒
  reconnectMaxTimes: 10                   // 最大重连次数，默认10次
});
```

### 2. 添加事件监听

```javascript
// 连接打开事件
WebSocketUtil.addEventListener('open', (event) => {
  console.log('WebSocket连接已建立');
});

// 消息接收事件
WebSocketUtil.addEventListener('message', ({ data }) => {
  console.log('收到消息:', data);
});

// 错误事件
WebSocketUtil.addEventListener('error', (event) => {
  console.log('WebSocket连接错误');
});

// 关闭事件
WebSocketUtil.addEventListener('close', (event) => {
  console.log('WebSocket连接已关闭');
});
```

### 3. 发送消息

```javascript
// 发送文本消息
WebSocketUtil.send('Hello, WebSocket!');

// 发送JSON对象
WebSocketUtil.send({
  type: 'chat',
  content: '这是一条消息',
  timestamp: new Date().getTime()
});
```

### 4. 关闭连接

```javascript
WebSocketUtil.close();
```

### 5. 获取连接状态

```javascript
const status = WebSocketUtil.getStatus();
// 0: 未连接, 1: 连接中, 2: 已连接
```

## 在Vue组件中使用

在Vue组件中使用WebSocket时，建议在组件创建时添加事件监听，在组件销毁前关闭连接：

```javascript
export default {
  created() {
    // 初始化WebSocket
    WebSocketUtil.init({
      url: 'ws://your-websocket-server.com'
    });
    
    // 添加事件监听
    WebSocketUtil.addEventListener('message', this.handleMessage);
  },
  beforeDestroy() {
    // 移除事件监听
    WebSocketUtil.removeEventListener('message', this.handleMessage);
    
    // 关闭连接
    WebSocketUtil.close();
  },
  methods: {
    handleMessage({ data }) {
      // 处理接收到的消息
      console.log('收到消息:', data);
    }
  }
};
```

## 示例组件

项目中提供了一个WebSocketDemo组件，可以通过访问`/websocket-demo`路由查看示例效果。

示例组件展示了如何在Vue组件中使用WebSocketUtil，包括：

1. 连接/断开WebSocket
2. 发送消息
3. 接收消息
4. 心跳检测
5. 状态显示

## 注意事项

1. WebSocket连接需要服务器支持，确保服务器已正确配置WebSocket
2. 心跳间隔不宜过短，建议30秒以上
3. 在组件销毁前务必关闭WebSocket连接，避免内存泄漏
4. 处理接收到的消息时，注意判断消息类型，避免错误处理
5. 在移动设备上，网络切换可能导致WebSocket连接断开，此时会自动重连