/**
 * WebSocket工具类
 * 功能：
 * 1. WebSocket连接管理
 * 2. 自动重连机制
 * 3. 心跳检测机制
 * 4. 消息发送和接收
 * 5. 事件监听和回调
 */

class WebSocketClient {
  /**
   * 构造函数
   * @param {Object} options 配置项
   * @param {String} options.url WebSocket连接地址
   * @param {Number} options.heartbeatInterval 心跳间隔时间(ms)，默认30000ms
   * @param {Number} options.reconnectInterval 重连间隔时间(ms)，默认5000ms
   * @param {Number} options.reconnectMaxTimes 最大重连次数，默认10次
   * @param {Function} options.onOpen 连接成功回调
   * @param {Function} options.onMessage 消息接收回调
   * @param {Function} options.onError 错误回调
   * @param {Function} options.onClose 连接关闭回调
   */
  constructor(options) {
    this.url = options.url || '';
    this.heartbeatInterval = options.heartbeatInterval || 30000; // 默认30s发送一次心跳
    this.reconnectInterval = options.reconnectInterval || 5000; // 默认5s重连一次
    this.reconnectMaxTimes = options.reconnectMaxTimes || 10; // 默认最多重连10次
    this.reconnectCount = 0; // 当前重连次数
    this.heartbeatTimer = null; // 心跳定时器
    this.reconnectTimer = null; // 重连定时器
    this.websocket = null; // WebSocket实例
    this.status = 0; // 连接状态：0-未连接，1-连接中，2-已连接
    this.onOpen = options.onOpen || function() {};
    this.onMessage = options.onMessage || function() {};
    this.onError = options.onError || function() {};
    this.onClose = options.onClose || function() {};
    
    // 初始化连接
    if (this.url) {
      this.connect();
    }
  }

  /**
   * 连接WebSocket
   * @param {String} url 连接地址，可选，如果不传则使用构造函数中的url
   */
  connect(url) {
    if (url) {
      this.url = url;
    }
    
    if (!this.url) {
      console.log('WebSocket连接地址不能为空');
      return;
    }
    
    // 防止重复连接
    if (this.status === 1) {
      console.log('WebSocket正在连接中，请勿重复连接');
      return;
    }
    
    // 如果已经连接，先关闭
    if (this.websocket) {
      this.close();
    }
    
    this.status = 1; // 标记为连接中
    
    try {
      this.websocket = new WebSocket(this.url);
      
      // 监听连接打开事件
      this.websocket.onopen = (event) => {
        this.status = 2; // 标记为已连接
        this.reconnectCount = 0; // 重置重连次数
        console.log('WebSocket连接成功');
        
        // 开启心跳检测
        this.heartbeat();
        
        // 执行外部回调
        this.onOpen(event);
      };
      
      // 监听消息接收事件
      this.websocket.onmessage = (event) => {
        // 收到任何消息都视为连接正常，重置心跳定时器
        this.resetHeartbeat();
        
        // 处理消息
        let data = event.data;
        try {
          // 尝试解析JSON
          data = JSON.parse(data);
          
          // 如果是心跳响应，不触发外部回调
          if (data.type === 'heartbeat') {
            console.log('收到心跳响应:', data);
            return;
          }
        } catch (e) {
          // 非JSON格式，保持原样
        }
        
        // 执行外部回调
        this.onMessage(data, event);
      };
      
      // 监听错误事件
      this.websocket.onerror = (event) => {
        console.log('WebSocket连接错误:', event);
        this.status = 0; // 标记为未连接
        
        // 执行外部回调
        this.onError(event);
        
        // 尝试重连
        this.reconnect();
      };
      
      // 监听关闭事件
      this.websocket.onclose = (event) => {
        console.log('WebSocket连接关闭:', event);
        this.status = 0; // 标记为未连接
        
        // 清除心跳定时器
        this.clearHeartbeat();
        
        // 执行外部回调
        this.onClose(event);
        
        // 如果不是主动关闭，则尝试重连
        if (event.code !== 1000) {
          this.reconnect();
        }
      };
    } catch (error) {
      console.log('WebSocket连接失败:', error);
      this.status = 0; // 标记为未连接
      
      // 尝试重连
      this.reconnect();
    }
  }

  /**
   * 重连
   */
  reconnect() {
    // 已达到最大重连次数
    if (this.reconnectCount >= this.reconnectMaxTimes) {
      console.warn(`WebSocket已达到最大重连次数${this.reconnectMaxTimes}次，停止重连`);
      return;
    }
    
    // 防止重复启动重连定时器
    if (this.reconnectTimer) {
      clearTimeout(this.reconnectTimer);
      this.reconnectTimer = null;
    }
    
    this.reconnectTimer = setTimeout(() => {
      console.log(`WebSocket第${this.reconnectCount + 1}次重连...`);
      this.reconnectCount++;
      this.connect(); // 重新连接
    }, this.reconnectInterval);
  }

  /**
   * 发送心跳
   */
  heartbeat() {
    // 清除之前的心跳定时器
    this.clearHeartbeat();
    
    // 创建新的心跳定时器
    this.heartbeatTimer = setInterval(() => {
      if (this.status === 2) {
        // 发送心跳消息
        this.send({
          type: 'heartbeat',
          data: 'ping',
          timestamp: new Date().getTime()
        });
        // console.log('发送心跳消息');
      } else {
        // 如果连接已断开，清除心跳定时器
        this.clearHeartbeat();
      }
    }, this.heartbeatInterval);
  }

  /**
   * 重置心跳
   * 收到消息后调用，重置心跳定时器
   */
  resetHeartbeat() {
    this.clearHeartbeat();
    this.heartbeat();
  }

  /**
   * 清除心跳定时器
   */
  clearHeartbeat() {
    if (this.heartbeatTimer) {
      clearInterval(this.heartbeatTimer);
      this.heartbeatTimer = null;
    }
  }

  /**
   * 发送消息
   * @param {Object|String} data 要发送的数据
   * @returns {Boolean} 是否发送成功
   */
  send(data) {
    if (this.status !== 2) {
      console.log('WebSocket未连接，无法发送消息');
      return false;
    }
    
    try {
      // 如果是对象，转为JSON字符串
      if (typeof data === 'object') {
        data = JSON.stringify(data);
      }
      
      this.websocket.send(data);
      return true;
    } catch (error) {
      console.log('WebSocket发送消息失败:', error);
      return false;
    }
  }

  /**
   * 关闭连接
   */
  close() {
    // 清除定时器
    this.clearHeartbeat();
    
    if (this.reconnectTimer) {
      clearTimeout(this.reconnectTimer);
      this.reconnectTimer = null;
    }
    
    // 关闭WebSocket连接
    if (this.websocket) {
      // 移除所有事件监听
      this.websocket.onopen = null;
      this.websocket.onmessage = null;
      this.websocket.onerror = null;
      this.websocket.onclose = null;
      
      // 如果连接已打开，则关闭
      if (this.status === 2) {
        this.websocket.close(1000, '主动关闭');
      }
      
      this.websocket = null;
    }
    
    this.status = 0; // 标记为未连接
  }

  /**
   * 获取当前连接状态
   * @returns {Number} 0-未连接，1-连接中，2-已连接
   */
  getStatus() {
    return this.status;
  }
}

// 导出WebSocket工具类
export default WebSocketClient;